# Define function to allocate specimens with indeterminate domestication status pro rata
taxa.pro.rata <- function(data, taxon.list, indet.taxon, default, groups, round=TRUE) {
    full.list <- c(taxon.list, indet.taxon)
    sum.all <- data[TAXON_GROUP%in%full.list, sum(NISP), by=groups]
    sum.indet <- data[TAXON_GROUP%in%indet.taxon, sum(NISP), by=groups]
    sum.all <- merge(sum.all, sum.indet, by=groups, all=TRUE)
    setnames(sum.all, old=c("V1.x", "V1.y"), new=c("all", "indet"))
    sum.all[is.na(all), all:=0]
    sum.all[is.na(indet), indet:=0]
    data <- merge(data, sum.all, by=groups, all=TRUE)
    data[TAXON_GROUP%in%taxon.list, NISP:=(NISP+((indet/(all-indet))*NISP))]
    data[all==indet & TAXON_GROUP==indet.taxon, wild_domestic_status:="domestic"]
    data[all==indet & TAXON_GROUP==indet.taxon, TAXON_GROUP:=default]
    data[,indet:=NULL]
    data[,all:=NULL]
    if(round==TRUE) {data[,NISP:=round(NISP)]}
    data[!TAXON_GROUP%in%indet.taxon,]
}

# Define function to estimate date ranges from data table of phases and dates
chron.range <- function(data, quants=c(0.15865, 0.5, 0.84135)) {
  if(nrow(data) == 1) {
    z <- with(data, BchronCalibrate(data$C14AGE, data$C14STD, data$curve))
    quantile(sample(z[[1]][[4]], size = 1000, replace=TRUE, prob=z[[1]][[5]]), quants, names=FALSE) - 1950
  } else {
    z <- with(data, BchronDensity(data$C14AGE, data$C14STD, data$curve))
    quantile(z[[1]], quants, names=FALSE) - 1950
  }
}  

# Define function for plotting custom beeswarms
NISPswarm <- function(data, metric, metric.lab, period.lab, col.list, col.by) {
    with(data, boxplot(get(metric) ~ ZONE,
                     outline=FALSE,
                     col=rgb(0.2, 0.2, 0.2, 0.2),
                     main=period.lab,
                     cex.lab=1.2,
                     cex.axis=1.2,
                     cex=1.2,
                     ylab=metric.lab,
                     ylim=c(0,100)))
    with(data, beeswarm(get(metric) ~ ZONE,
                      pch=21,
                      cex=1.2,
                      spacing=1.5,
                      pwpch=type.symbols[TYPE],
                      pwbg=col.list[get(col.by)],
                      add=TRUE))
}
